--[[
ShadowsFog LuCI Configuration Page.
References:
https://github.com/
--]]


local fs   = require "nixio.fs"
local SYS  = require "luci.sys"
local conf = "/etc/shadowfog/dnslist.pac"
local err_file = "/etc/shadowfog_error_file"

local err_msg      = ""
local state_msg    = ""
local ss_router_on = ""

ss_router_on = (luci.sys.call("pidof ss-router > /dev/null") == 0)

if ss_router_on then
        state_msg = "<b><font color=\"green\">" .. translate("Running") .. "</font></b>"
else
        state_msg = "<b><font color=\"red\">" .. translate("Not running") .. "</font></b>"
end

err_msg = fs.readfile(err_file) or ""


m = Map("shadowfog", translate("ShadowFog Transparent Proxy"),
        translate("A fast secure tunnel proxy that help you get through firewalls on your router")
        .. "<a target=\"_blank\" href=\"http://shadowfog.com\">" .. translate("Not have an account").. "</a>"
        .. "<br/><br/>" .. translate("Run Status").. " : "  .. state_msg .. "<br/>"
        .. "<br/>" .. translate(err_msg) .. "<br/>")

s = m:section(TypedSection, "shadowfog")
s.anonymous = true

-- ---------------------------------------------------

s:tab("basic",  translate("Basic"))


switch = s:taboption("basic",Flag, "enabled", translate("Enable"))
switch.rmempty = false

switch = s:taboption("basic",Value, "shadowfog_user", translate("Username"))
shadowfog_passwd = s:taboption("basic",Value, "shadowfog_passwd", translate("Password"))
shadowfog_passwd.password = true

proxy_mode = s:taboption("basic",ListValue, "proxy_mode", translate("Proxy Mode"))
proxy_mode:value("M", translate("GFW-List auto proxy"))
proxy_mode:value("G", translate("global proxy"))

--[[
s:tab("main",  translate("Advance"))
switch = s:taboption("main",Flag, "advance_mode", translate("Enable"))
switch.rmempty = false

server_ip = s:taboption("main",Value, "server_ip", translate("Server Address"))
server_ip.datatype = "host"

server_port = s:taboption("main",Value, "server_port", translate("Server Port"))
server_port.datatype = "range(1,65535)"

server_passwd = s:taboption("main",Value, "server_passwd", translate("Password"))
server_passwd.password = true

method = s:taboption("main",ListValue, "method", translate("Encryption Method"))
method:value("table")
method:value("rc4")
method:value("rc4-md5")
method:value("aes-128-cfb")
method:value("aes-192-cfb")
method:value("aes-256-cfb")
method:value("bf-cfb")
method:value("camellia-128-cfb")
method:value("camellia-192-cfb")
method:value("camellia-256-cfb")
method:value("cast5-cfb")
method:value("des-cfb")
method:value("idea-cfb")
method:value("rc2-cfb")
method:value("seed-cfb")
method:value("salsa20")
method:value("chacha20")
method:value("chacha20-ietf")
--]]


s:tab("list",  translate("manual GFW-List"))
gfwlist = s:taboption("list", TextValue, "conf")
gfwlist.description = translate("")
gfwlist.rows = 13
gfwlist.wrap = "off"

gfwlist.cfgvalue = function(self, section)
        return fs.readfile(conf) or ""
end

gfwlist.write = function(self, section, value)
        fs.writefile(conf, value:gsub("\r\n", "\n"))
end

return m
