/*
 *	shadowfog_restart.c
 *
 *  fuqiyang f@pear.hk 2017.04.19
 *
**/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <errno.h>
#include "jansson.h"


// ================  data struct ============================
//
enum task_mode {

	TASK_MODE_NONE,

	TASK_MODE_REPEAT,
};

enum task_pid {
	
	TASK_PID_ZERO = 0,
};

#define PEAR_UPDATE_CONF_FILE                   "/etc/shadowfog/update.json"
#define TASK_NAME_STR_LEN						64

#define TASK_ARGUMENT_COUNT						8

#define TASK_ARGUMENT_STR_LEN					32

#define TASK_COUNT								32

#define TASK_PATH_WORKING_DIR_DEFAULT			"/etc/shadowfog"

#define TASK_PATH_WORKING_DIR_STR_LEN			32

#define TASK_PATH_CONFIGURE_FILE_DEFAULT		TASK_PATH_WORKING_DIR_DEFAULT"/restart.json"

#define TASK_PATH_CONFIGURE_FILE_STR_LEN		64


typedef struct {
	
	int 	task_mode;

	pid_t	task_pid;

	char	task_name[TASK_NAME_STR_LEN];

	char	task_argument[TASK_ARGUMENT_COUNT][TASK_ARGUMENT_STR_LEN];

}shadowfog_task_info_t;


typedef struct {

	shadowfog_task_info_t	task_info[TASK_COUNT];

	int					task_info_size;	

	char 				task_path_working_dir[TASK_PATH_WORKING_DIR_STR_LEN];

	char				task_path_configure_file[TASK_PATH_CONFIGURE_FILE_STR_LEN];

	int 				stdout_stderr_fd;	// open /dev/null to redirect stdout and stderr

}shadowfog_task_info_manager_t;


static
shadowfog_task_info_manager_t	shadowfog_task_info_manager;


#define TASK_MODE(task_index)								shadowfog_task_info_manager.task_info[task_index].task_mode

#define TASK_MODE_INIT(task_index)							shadowfog_task_info_manager.task_info[task_index].task_mode = TASK_MODE_NONE

#define TASK_MODE_SET(task_index, mode)						shadowfog_task_info_manager.task_info[task_index].task_mode = mode

#define TASK_PID(task_index)								shadowfog_task_info_manager.task_info[task_index].task_pid

#define TASK_PID_INIT(task_index)							shadowfog_task_info_manager.task_info[task_index].task_pid = TASK_PID_ZERO

#define TASK_NAME(task_index)								shadowfog_task_info_manager.task_info[task_index].task_name

#define TASK_NAME_INIT(task_index)							shadowfog_task_info_manager.task_info[task_index].task_name[0] = '\0'

#define TASK_NAME_SET_STRING(task_index, str)				strcpy(shadowfog_task_info_manager.task_info[task_index].task_name, str)

#define TASK_ARGUMENT(task_index)							shadowfog_task_info_manager.task_info[task_index].task_argument

#define TASK_ARGUMENT_INIT(task_index)						do {													\
																	int i;											\
																	for (i = 0; i < TASK_ARGUMENT_COUNT; ++i) {		\
																													\
																		TASK_ARGUMENT(task_index)[i][0] = '\0';		\
																	}												\
															}while(0)

#define TASK_ARGUMENT_SET_NULL(task_index)					TASK_ARGUMENT_INIT(task_index)

#define TASK_ARGUMENT_IS_NULL(task_index)					(shadowfog_task_info_manager.task_info[task_index].task_argument[0][0] == '\0')

#define TASK_ARGUMENT_SET_STRING(task_index, index, str)	strcpy(shadowfog_task_info_manager.task_info[task_index].task_argument[index], str)

#define TASK_ARGUMENT_GET_STRING(task_index, index)			(TASK_ARGUMENT(task_index)[index])

#define TASK_ARGUMENT_STRING_IS_NOT_NULL(task_index, index) (shadowfog_task_info_manager.task_info[task_index].task_argument[index][0] != '\0')

#define TASK_INFO											shadowfog_task_info_manager.task_info

#define TASK_INFO_INIT(task_index)							do {									\
																	TASK_MODE_INIT(task_index);		\
																	TASK_PID_INIT(task_index);		\
																	TASK_NAME_INIT(task_index);		\
																	TASK_ARGUMENT_INIT(task_index);	\
															} while(0)

#define TASK_INFO_SIZE										shadowfog_task_info_manager.task_info_size

#define TASK_INFO_SIZE_INIT()								TASK_INFO_SIZE = 0

#define TASK_INFO_SIZE_INCREMENT()							++TASK_INFO_SIZE

#define TASK_PATH_WORKING_DIR								shadowfog_task_info_manager.task_path_working_dir

#define TASK_PATH_WORKING_DIR_INIT(path)					sprintf(shadowfog_task_info_manager.task_path_working_dir, \
																								TASK_PATH_WORKING_DIR_DEFAULT)

#define TASK_PATH_CONFIGURE_FILE							shadowfog_task_info_manager.task_path_configure_file

#define TASK_PATH_CONFIGURE_FILE_INIT()						sprintf(shadowfog_task_info_manager.task_path_configure_file, \
																								TASK_PATH_CONFIGURE_FILE_DEFAULT)
#define TASK_STDOUT_STDERR_FD								shadowfog_task_info_manager.stdout_stderr_fd

#define PEAR_TASK_NEED_START_TASK(task_index)				(TASK_PID(task_index) == 0 && TASK_MODE(task_index) == TASK_MODE_REPEAT)

#define PEAR_TASK_CAN_KILL_TASK(task_index)					(TASK_PID(task_index) != 0 && TASK_MODE(task_index) != TASK_MODE_NONE)

#define PEAR_TASK_FINISH_UPGRADE(pid, status)				(pid == TASK_PID(0) && WIFEXITED(status) && 0 == WEXITSTATUS(status))

#define PEAR_TASK_HAVE_ABORT_TASK(task_pid) 				(task_pid > 0)



// ================ operate task_info ======================
//

static
void shadowfog_task_init_task_manager_structure(void)
{
	int task_index;

	for (task_index = 0; task_index < TASK_COUNT; ++task_index) {

		TASK_INFO_INIT(task_index);
	}

	TASK_INFO_SIZE_INIT();

	TASK_PATH_WORKING_DIR_INIT();

	TASK_PATH_CONFIGURE_FILE_INIT();

	// internal set /usr/sbin/shadowfog_update
	//
	TASK_NAME_SET_STRING(0, "shadowfog_update");

	TASK_MODE_SET(0, TASK_MODE_REPEAT);
    TASK_ARGUMENT_SET_STRING(0, 0, "-d");
    TASK_ARGUMENT_SET_STRING(0, 1, "-c");
    TASK_ARGUMENT_SET_STRING(0, 2, PEAR_UPDATE_CONF_FILE);
}


// ================ kill task ======================
//
static
void shadowfog_task_kill_all_programs(void)
{
	int task_index;

	fprintf(stderr, "try to kill all program: \n");

	for(task_index = 0; task_index < TASK_COUNT; ++task_index) {

		if (PEAR_TASK_CAN_KILL_TASK(task_index)) {

			kill(TASK_PID(task_index), SIGKILL);

			fprintf(stderr, "kill %s\n", TASK_NAME(task_index));
		}

		TASK_PID(task_index) = 0;
	}
	fprintf(stderr, "finish trying...\n");
}


static
void shadowfog_task_init_pid_because_task_dead(int task_pid)
{
	int task_index;

	for(task_index = 0; task_index < TASK_COUNT; ++task_index){

		if(TASK_PID(task_index) == task_pid) { TASK_PID(task_index) = 0; }
	}
}


// ================ parse configure file ===================
//
static
void shadowfog_task_parse_configure_file(json_t *json_root)
{
	json_t *json_tasks_array;

	json_tasks_array = json_object_get(json_root, "tasks");

	// check "tasks":[{}, {}, {}]
	//
	if (json_tasks_array == NULL || !json_is_array(json_tasks_array)) {

		fprintf(stderr, "configure file error\n");

		return;
	}

	size_t task_index;

	json_t *task_value;

	// travel all {"mode":"xx", "name":"xx", "argument":["xx1", "xx2"]} in "tasks":[]
	//
	json_array_foreach(json_tasks_array, task_index, task_value) {

		// parse name
		//
		json_t *task_name = json_object_get(task_value, "name");

		if (task_name == NULL || !json_is_string(task_name)) {
			exit(-1);
		}

		TASK_NAME_SET_STRING(task_index + 1, json_string_value(task_name));

		// parse mode
		//
		json_t *task_mode = json_object_get(task_value, "mode");

		TASK_MODE_SET(task_index + 1, TASK_MODE_NONE);

		if (task_mode == NULL || !json_is_string(task_mode)) {
		
			TASK_MODE_SET(task_index + 1, TASK_MODE_REPEAT);

		} else {

			if (strcmp(json_string_value(task_mode), "repeat") == 0) {

				TASK_MODE_SET(task_index + 1, TASK_MODE_REPEAT);
			}
		}

		// parse argument
		//
		json_t *task_argument_array = json_object_get(task_value, "argument");

		if (task_argument_array == NULL || !json_is_array(task_argument_array)) {

			TASK_ARGUMENT_SET_NULL(task_index + 1);

		} else {

			size_t task_argument_index;

			json_t *task_argument_value;

			json_array_foreach(task_argument_array, task_argument_index, task_argument_value) {

				TASK_ARGUMENT_SET_STRING(task_index + 1, task_argument_index, json_string_value(task_argument_value));
			}
		}

		// add a task and init pid
		//
		TASK_INFO_SIZE_INCREMENT();

		TASK_PID_INIT(task_index + 1);
	}

	json_decref(json_root);
}


static
json_t *shadowfog_task_check_configure_file(void)
{
	// read json data from configure file
	//
	char json_buf[10240];

	FILE *file = fopen(TASK_PATH_CONFIGURE_FILE, "r");

	if (NULL == file) { perror("fopen"); exit(-1); }

	int readBytes = fread(json_buf, 1, sizeof(json_buf) - 1, file);

	json_buf[readBytes] = '\0';

	fclose(file);

	// check json_root
	//
	json_t *json_root;

	json_error_t error;

	json_root = json_loads(json_buf, 0, &error);

	return json_root;
}


static
void shadowfog_task_period_parse_configure_file(void)
{
	json_t *json_root = shadowfog_task_check_configure_file();

	if (NULL == json_root) {

		fprintf(stderr, "configure file error\n");

		return;
	}

	shadowfog_task_parse_configure_file(json_root);
}


// ================ birth task manager =====================
//
static
void shadowfog_task_execvp(int task_index)
{
	int argument_index = 0;

	int argv_index = 0;

	char *argv[TASK_ARGUMENT_COUNT];
	
	argv[argv_index++] = TASK_NAME(task_index);

	for (argument_index = 0; TASK_ARGUMENT_STRING_IS_NOT_NULL(task_index, argument_index); ++argument_index) {

		argv[argv_index++] = (char *)TASK_ARGUMENT_GET_STRING(task_index, argument_index);
	}
	argv[argv_index++] = NULL;

    fprintf(stderr, "TASK_NAME:%s", TASK_NAME(task_index));
    fprintf(stderr, "argv[0]:%s", argv[0]);

	execvp(TASK_NAME(task_index), argv);
}


static
void shadowfog_task_birth_task_manager(void)
{
	int task_index;

	for (task_index = 0; task_index < TASK_COUNT; ++task_index) {

		if (PEAR_TASK_NEED_START_TASK(task_index)) {

			int task_pid = fork();

			if (task_pid < 0) {

				perror("fork");

				return;
			}

			// child process
			//
			if (0 == task_pid) {

				shadowfog_task_execvp(task_index);

				// fprintf(stderr, "execlp(%s, %s, 0): %s\n",
				// 		TASK_NAME(task_index), 
				// 		TASK_NAME(task_index), 
				// 		strerror(errno));

				exit(-1);

			// parent process
			//
			} else { TASK_PID(task_index) = task_pid; }
		}
	}
}


// ================ death task manager =====================
//
static
int shadowfog_task_death_task_manager(void)
{
	int status;

	int task_pid = waitpid(-1, &status, WNOHANG);

	// when child process die out, it is true 
	// 
	if (PEAR_TASK_HAVE_ABORT_TASK(task_pid)) {

		// if node-monitor exit(0), means finish upgrading, restart all the programs
		//
		if (PEAR_TASK_FINISH_UPGRADE(task_pid, status)) {

			shadowfog_task_kill_all_programs();

			return 1;

		// otherwise set dead process's pid zero, wait for restart in shadowfog_task_birth_task_manager()
		//
		} else {

			shadowfog_task_init_pid_because_task_dead(task_pid);

			return 2;
		}
	}
	return 0;
}


int main(int argc, char *argv[])
{
    daemon(0, 0);
restart_program:

	// redirect
	//
	//TASK_STDOUT_STDERR_FD = open("/dev/null", O_WRONLY);

	//if (TASK_STDOUT_STDERR_FD < 0) {

	//	exit(1);
	//}

	//dup2(TASK_STDOUT_STDERR_FD, 1);

	//dup2(TASK_STDOUT_STDERR_FD, 2);

	shadowfog_task_init_task_manager_structure();

    printf("argc:%d\n", argc);
    if (argc == 3)
    {
        printf("argv0:%s\n argv1:%s\n argv2:%s\n",  argv[0], argv[1], argv[2]);
        sprintf(shadowfog_task_info_manager.task_path_configure_file, "%s", argv[1]);
        TASK_ARGUMENT_SET_STRING(0, 2, argv[2]);
    }
	shadowfog_task_period_parse_configure_file();

	while (1) {

		shadowfog_task_birth_task_manager();

		if (1 == shadowfog_task_death_task_manager()) {

			fprintf(stdout, "remote update success...\n");

			close(TASK_STDOUT_STDERR_FD);

			goto restart_program;
		}

		sleep(5);
	}

	return 0;
}


