/*
 * jconf.c - Parse the JSON format config file
 *
 * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
 *
 * This file is part of the shadowsocks-libev.
 * shadowsocks-libev is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * shadowsocks-libev is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with shadowsocks-libev; see the file COPYING. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "utils.h"
#include "jconf.h"
#include "jansson.h"

#include <libcork/core.h>


//char * to_string(const json_value *value)
//{
//    if (value->type == json_string) {
//        return ss_strndup(value->u.string.ptr, value->u.string.length);
//    } else if (value->type == json_integer) {
//        return strdup(ss_itoa(value->u.integer));
//    } else if (value->type == json_null) {
//        return "null";
//    } else {
//        LOGE("%d", value->type);
//        FATAL("Invalid config format.");
//    }
//    return 0;
//}

void free_addr(ss_addr_t *addr)
{
    ss_free(addr->host);
    ss_free(addr->port);
}

void
parse_addr(const char *str, ss_addr_t *addr)
{
    int ipv6 = 0, ret = -1, n = 0;
    char *pch;

    struct cork_ip ip;
    if (cork_ip_init(&ip, str) != -1) {
        addr->host = strdup(str);
        addr->port = NULL;
        return;
    }

    pch = strchr(str, ':');
    while (pch != NULL) {
        n++;
        ret = pch - str;
        pch = strchr(pch + 1, ':');
    }
    if (n > 1) {
        ipv6 = 1;
        if (str[ret - 1] != ']') {
            ret = -1;
        }
    }

    if (ret == -1) {
        if (ipv6) {
            addr->host = ss_strndup(str + 1, strlen(str) - 2);
        } else {
            addr->host = strdup(str);
        }
        addr->port = NULL;
    } else {
        if (ipv6) {
            addr->host = ss_strndup(str + 1, ret - 2);
        } else {
            addr->host = ss_strndup(str, ret);
        }
        addr->port = strdup(str + ret + 1);
    }
}


jconf_t * read_jconf(const char *json_buffer)
{
    static jconf_t conf;

    memset(&conf, 0, sizeof(jconf_t));

    // jansson
    //
    json_error_t error;

    json_t *json_root = json_loads(json_buffer, 0, &error);

    if (json_root == NULL) { return NULL; }

    // fogvdn_force_update
    //
    json_t *json_key = json_object_get(json_root, "fogvdn_force_update");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_force_update, json_string_value(json_key));
    }

    // fogvdn_host
    //
    json_key = json_object_get(json_root, "fogvdn_host");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_host, json_string_value(json_key));
    }

    // fogvdn_signal_ws
    //
    json_key = json_object_get(json_root, "fogvdn_signal_ws");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_signal_ws, json_string_value(json_key));
    }

    // fogvdn_signal_wss
    //
    json_key = json_object_get(json_root, "fogvdn_signal_wss");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_signal_wss, json_string_value(json_key));
    }

    // fogvdn_version
    //
    json_key = json_object_get(json_root, "fogvdn_version");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_version, json_string_value(json_key));
    }

    // fogvdn_url
    //
    json_key = json_object_get(json_root, "fogvdn_url");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_url, json_string_value(json_key));
    }

    // fogvdn_md5
    //
    json_key = json_object_get(json_root, "fogvdn_md5");

    if (json_key && json_is_string(json_key)) {

        strcpy(conf.fogvdn_md5, json_string_value(json_key));
    }

    //
    //
    json_decref(json_root);

    return &conf;
}
