#include "shadowfog_event.h"
#include "shadowfog_update.h"

#ifndef BUF_SIZE
#define BUF_SIZE 65535
#endif


// ----------------------- init  -------------------------
//
struct ev_loop *loop;


void shadowfog_event_init(void)
{
	loop = EV_DEFAULT;
}


// ----------------------- signal  -------------------------
//
struct ev_signal sigint_watcher;
struct ev_signal sigterm_watcher;


static void signal_cb_update(EV_P_ ev_signal *w, int revents)
{
    if (revents & EV_SIGNAL) {
        switch (w->signum) {
            case SIGINT:
            case SIGTERM:
                ev_unloop(EV_A_ EVUNLOOP_ALL);
                break;

            default:
                break;
        }
    }
}


void shadowfog_event_set_signals_for_update(void)
{
    ev_signal_init(&sigint_watcher, signal_cb_update, SIGINT);
    ev_signal_init(&sigterm_watcher, signal_cb_update, SIGTERM);
    ev_signal_start(loop, &sigint_watcher);
    ev_signal_start(loop, &sigterm_watcher);
}


// ----------------------- update timer  -------------------------
//
ev_timer shadowfog_timeout_update_watcher;


void shadowfog_event_update(void)
{
    shadowfog_update(shadowfog_update_info.curl,
                shadowfog_machine_info.host_ip,
                shadowfog_machine_info.mac,
                shadowfog_machine_info.extern_ip,
                shadowfog_update_info.package,
                shadowfog_update_info.version);
}


void shadowfog_event_set_update_timer(void)
{
    srandom(time(NULL));

//                    random() % PEAR_TIMEOUT_UPDATE_OUTSIDE,
    ev_timer_init (&shadowfog_timeout_update_watcher,
                    shadowfog_event_update,
                    0,
                    random() % PEAR_TIMEOUT_UPDATE_OUTSIDE + PEAR_TIMEOUT_UPDATE_OUTSIDE);

    ev_timer_start (loop, &shadowfog_timeout_update_watcher);
}


// ----------------------- run and stop  -------------------------
//
void shadowfog_event_run(void)
{
	ev_run(loop, 0);
}


void shadowfog_event_stop(void)
{
	ev_signal_stop(EV_DEFAULT, &sigint_watcher);
    
    ev_signal_stop(EV_DEFAULT, &sigterm_watcher);
}






