
#include <unistd.h>

#ifndef __MINGW32__

#include <netdb.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <pthread.h>
#include <sys/un.h>
#include <sys/socket.h>

#endif

#include <net/if.h>

#include <sys/ioctl.h>

#include <sys/sysinfo.h>

#include "shadowfog_machine.h"

#include "shadowfog_utils.h"


shadowfog_machine_info_t shadowfog_machine_info;


typedef struct {
    char  * url;
    unsigned char type;
} shadowfog_external_ip_t;


shadowfog_external_ip_t shadowfog_external_ip[]={
    {.url="http://whatismyip.akamai.com",.type=PEAR_EXTERNAL_HTTP_RAW},
    {.url="http://members.3322.org/dyndns/getip",.type=PEAR_EXTERNAL_HTTP_RAW},
    {.url="http://ipinfo.io/json",.type=PEAR_EXTERNAL_HTTP_JSON},
    {.url="http://myexternalip.com/json",.type=PEAR_EXTERNAL_HTTP_JSON},
    {.url="https://api.ipify.org/?format=json",.type=PEAR_EXTERNAL_HTTP_JSON},
};


void shadowfog_machine_info_init(void)
{
	memset(&shadowfog_machine_info, 0, sizeof(shadowfog_machine_info_t));

    shadowfog_machine_info.max_server = PEAR_MAX_SS_SERVER;

    strcpy(shadowfog_machine_info.working_dir, WORKING_DIR);
    
    detect_hardware_platform();         // sprintf(shadowfog_machine_info.platform, "%s", PLATFORM);

    // get system info
    //
    shadowfog_machine_get_external_ip(shadowfog_machine_info.extern_ip);

    shadowfog_machine_get_local_ip(shadowfog_machine_info.local_ip, sizeof(shadowfog_machine_info.local_ip));

    shadowfog_machine_get_mac(shadowfog_machine_info.mac);
}


void shadowfog_machine_get_external_ip(char * value)
{
    shadowfog_ddns_req_t req;
    shadowfog_ddns_rsp_t rsp;
    shadowfog_ddns_conf_t conf;
    req.conf = &conf;
    shadowfog_ddns_init_conf(req.conf);
    req.query_str[0] = '\0';

   int url_size=sizeof(shadowfog_external_ip)/sizeof(shadowfog_external_ip[0]);
   for(int i=0;i<url_size;i++) {
   		// PEAR_LOG(MAIN, INFO, "[fogvdn]%d %s \r\n", url_size, shadowfog_external_ip[i].url);
        strncpy(req.api_url, shadowfog_external_ip[i].url,sizeof(req.api_url));
        shadowfog_ddns_query(&req, &rsp);
	  	if (rsp.size == 0) {
	    	continue;
      	}
   		// PEAR_LOG(MAIN, INFO, "[fogvdn]rsp.memory %s and length is %d %x\r\n",
					// 	rsp.memory,
					// 	strlen(rsp.memory),
					// 	rsp.memory[strlen(rsp.memory)+1]);								
	  	if(shadowfog_external_ip[i].type==PEAR_EXTERNAL_HTTP_RAW) {
	  		if(rsp.memory[strlen(rsp.memory)-1]==0x0A) {
	  	    	rsp.memory[strlen(rsp.memory)-1]=0x00;
	  		}	
	  		strcpy(value, rsp.memory);
	  	} else {
	  		if(shadow_get_json(rsp.memory, "ip", value) != 0) { strcpy(value, "0.0.0.0"); } 
	  	}
	  	free(rsp.memory);
	  	break;
   }
}


void shadowfog_machine_get_local_ip(char *buf, int buflen)
{
    struct sockaddr_in addr;
    socklen_t len;
    int fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd == -1)
        goto error;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(1);
    inet_pton(AF_INET, "1.2.3.4", &addr.sin_addr.s_addr);
    if (connect(fd, (const struct sockaddr*) &addr, sizeof(addr)) < 0)
        goto error;
    len = sizeof(addr);
    if (getsockname(fd, (struct sockaddr*) &addr, &len) < 0)
        goto error;
    if (getnameinfo((const struct sockaddr*) &addr, sizeof(addr),
                    buf, buflen,
                    NULL, 0, NI_NUMERICHOST) != 0)
        goto error;
    close(fd);
    return;
error:
    if (fd != -1)
        close(fd);

    strcpy(buf, "127.0.0.1");
}



static void shadowfog_string_tolower(char *dest, char *src)
{
    while (*src) {

        *dest++ = tolower(*src++);
    }
}


int  shadowfog_machine_get_mac(char * mac_address)
{
// newifi platform
//
#if (PLATFORM_TYPE == 6)

    char cmd[] = "ubus call xapi.basic get_lan_mac";
    char temp[256];
    memset(temp, 0, 256);

    FILE *read_file = popen(cmd, "r");
    fread(temp, 1, 256, read_file);
    pclose(read_file);
    PEAR_LOG(MAIN, INFO, "ubus call xapi.basic get_lan_mac: %s", temp);

    shadow_get_json(temp, "macaddr", shadowfog_machine_info.mac);
    shadowfog_string_tolower(shadowfog_machine_info.mac, shadowfog_machine_info.mac);

#else

    struct ifreq ifr;
    struct ifconf ifc;
    char buf[1024];
    int success = 0;

    int sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (sock == -1) { /* handle error*/ };

    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (ioctl(sock, SIOCGIFCONF, &ifc) == -1) { /* handle error */ }

    struct ifreq* it = ifc.ifc_req;
    const struct ifreq* const end = it + (ifc.ifc_len / sizeof(struct ifreq));

    for (; it != end; ++it) {
        strcpy(ifr.ifr_name, it->ifr_name);
        if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0) {
            if (! (ifr.ifr_flags & IFF_LOOPBACK)) { // don't count loopback
                if (ioctl(sock, SIOCGIFHWADDR, &ifr) == 0) {
                    success = 1;
                    break;
                }
            }
        } else { /* handle error */ }
    }
    unsigned char mac[6];
    if (success) {
        memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
        // sprintf(mac_address, "%C:%C:%C:%C:%C:%C", str[0], str[1], str[2], str[3], str[4], str[5]);
        sprintf(mac_address, "%02x:%02x:%02x:%02x:%02x:%02x", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    }

#endif
    return 0;
}



// get from man rand
static unsigned long next = 1;

/* RAND_MAX assumed to be 32767 */
int myrand(void)
{
    next = next * 1103515245 + 12345;
    return ((unsigned)(next / 65536) % 32768);
}

void mysrand(unsigned int seed)
{
    next = seed;
}

char shadowfog_lower_case [] = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k'};
char shadowfog_upper_case [] = {'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V'};
char shadowfog_special [] = {'#', '@', '$', '&', '*', '#', '@', '$', '*', '&', '?'};


/*
* Get the server info.
* l@pear.hk
*
*/
int shadowfog_get_host(void)
{
//#if DEBUG_LESS_FEATURE
    //strcpy(shadowfog_machine_info.host_ip, "http://192.168.1.130:8090/v1");
    //shadowfog_token_get_token("http://192.168.1.130:8090/v1");
    //return 0;
//#endif
    char temp[4096] = {0x00};
    char base64_temp[1024] = {0x00};
    long retcode;
    jconf_t *conf = NULL;

    int item = 0;

    int items = get_host_address_number();

    for (item = 0; item < items; item++) {

        char tmp_url[256];

        sprintf(tmp_url, "%sshadowfog_%s.json", shadowfog_host_address[item], PLATFORM);

        PEAR_LOG(MAIN, DEBUG, "items: %d, item: %d : get host cmd: %s, curl %s", items, item, shadowfog_host_address[item], tmp_url);

        char header_content_type[64] = "Content-Type: application/json";
        char header_cache_control[64] = "Cache-Control: no-cache";
        char *header[] = { &header_content_type, &header_cache_control, NULL };

        memset(temp, 0x00, 4096);

        memset(base64_temp, 0x00, 1024);

        retcode = shadowfog_curl_cmd("GET", tmp_url, NULL, header, 0, 0, temp, NULL);

        if (retcode != 200) {

            PEAR_LOG(MAIN, ERR, "Fail to connect to %s.", shadowfog_host_address[item]);

            continue;
        }
        PEAR_LOG(MAIN, INFO, "Success to connect to %s.", shadowfog_host_address[item]);

        PEAR_LOG(MAIN, DEBUG, "afert base64 decode the content: %s", temp);
        Base64decode(base64_temp, temp);

        PEAR_LOG(MAIN, DEBUG, "afert base64 decode the content: %s", base64_temp);

        conf = read_jconf(base64_temp);

        if (strlen(conf->fogvdn_host)) {

            strcpy(shadowfog_machine_info.host_ip, conf->fogvdn_host);

            return 0;

        } else { PEAR_LOG(MAIN, DEBUG, "conf->fogvdn_host is null string"); }
    }

    PEAR_LOG(MAIN, ERROR, "shadowfog_get_host exit(-1)");

    exit(-1);
}


