// yongming.li pear.hk
#ifndef _UTILS_H
#define _UTILS_H

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#define PORTSTRLEN 16
#define SS_ADDRSTRLEN (INET6_ADDRSTRLEN + PORTSTRLEN + 1)


#define STR(x) # x
#define TOSTR(x) STR(x)


#include <syslog.h>

extern int use_tty;
#define USE_TTY()                        \
    do {                                 \
        use_tty = isatty(STDERR_FILENO); \
    } while (0)                          \

#define HAS_SYSLOG
extern int use_syslog;

#define TIME_FORMAT "%F %T"

#define USE_SYSLOG(ident)                          \
    do {                                           \
        use_syslog = 1;                            \
        openlog((ident), LOG_CONS | LOG_PID, 0); } \
    while (0)

// added log level by  yongming.li  pear.hk
	
#define	EMERG	           0	                            /* system is unusable			*/
#define	ALERT	           1	                            /* action must be taken immediately	*/
#define	CRIT	           2	                            /* critical conditions			*/
#define	ERR	               3	                            /* error conditions			*/
#define	WARNING	    	   4	                            /* warning conditions			*/
#define	NOTICE	           5	                            /* normal but significant condition	*/
#define	INFO	           6	                            /* informational			*/
#define	DEBUG	           7	                            /* debug-level messages			*/
#define PEAR_LOGLEVEL     (7)
#define PEAR_LOG_LIMIT_SIZE (4 << 20)

/* Debug message event */
#define DBG_EVT_NONE                   0x00000000	
#define DBG_EVT_UPDATE                 0x00000001	
#define DBG_EVT_MAPPING               0x00000002	
#define DBG_EVT_SPEED                    0x00000004	
#define DBG_EVT_TOKEN                   0x00000008
#define DBG_EVT_INFO                      0x00000010	
#define DBG_EVT_SECRET                  0x00000020	
#define DBG_EVT_CACHE                    0x00000040	
#define DBG_EVT_TRAFFIC                 0x00000080	
#define DBG_EVT_MSC                       0x00000100
#define DBG_EVT_MAIN                     0x00000200
#define DBG_EVT_PARSE                     0x00000400
#define DBG_EVT_ALL                        0xffffffff
#define PEAR_EVT_MASK    (DBG_EVT_MAPPING | DBG_EVT_UPDATE | DBG_EVT_SPEED | DBG_EVT_MAIN | DBG_EVT_TOKEN | DBG_EVT_SECRET | DBG_EVT_PARSE)

void shadowfog_log_file(const char *msg);
extern char shadowfog_log_temp[];

#define PEAR_LOG(evt,level,format, ...)                                                        \
do {                                                                         \
    if (use_syslog) {                                                        \
        syslog(LOG_INFO, format, ## __VA_ARGS__);                            \
    } else {                                                                 \
    	if (level <= PEAR_LOGLEVEL){     \
			if ((DBG_EVT_##evt) & PEAR_EVT_MASK) { \
				const char * feature = #evt; \
			    time_t now = time(NULL);                                             \
			    char timestr[20];                                                  \
			    strftime(timestr, 20, TIME_FORMAT, localtime(&now));                 \
			    if (use_tty) {                                                       \
					if (level == DEBUG){     \
						fprintf(stderr, "\e[01;32m %s %s %s %d %s %s[%d]: \e[0m" format "\n",timestr,__FILE__,__FUNCTION__ ,__LINE__,feature, #level , level ,\
			    	             ## __VA_ARGS__);                                         \
					}  \
					else  \
					{  \
					    fprintf(stderr, "\e[01;32m %s %s %d %s %s[%d]: \e[0m" format "\n",timestr,__FILE__ ,__LINE__,feature, #level , level ,\
			    	             ## __VA_ARGS__);                                         \
					    sprintf(shadowfog_log_temp, "%s %s %d %s %s[%d]: " format "\n",timestr,__FILE__ ,__LINE__,feature, #level , level ,\
			    	             ## __VA_ARGS__);                                         \
					    shadowfog_log_file(shadowfog_log_temp);   \
					}  \
			    } else {                                                             \
			        fprintf(stderr, " %s INFO: " format "\n", timestr,               \
			                ## __VA_ARGS__);                                         \
			    }                                                                    \
	        } \
    	}\
    }                                                                        \
}                                                                            \
while (0)
		
// modified by yongming.li for for details (file,line)
#define LOGI(format, ...)                                                         \
    do {                                                                          \
        if (use_syslog) {                                                         \
            syslog(LOG_ERR, format, ## __VA_ARGS__);                              \
        } else {                                                                  \
            time_t now = time(NULL);                                              \
            char timestr[20];                                                     \
            strftime(timestr, 20, TIME_FORMAT, localtime(&now));                  \
            if (use_tty) {                                                        \
                fprintf(stderr, "\e[01;35m %s %s %d INFO: \e[0m" format "\n", timestr,__FILE__, __LINE__, \
                        ## __VA_ARGS__);                                          \
            } else {                                                              \
                fprintf(stderr, " %s INFO: " format "\n", timestr,               \
                        ## __VA_ARGS__);                                          \
            }                                                                     \
        }                                                                 \
    }                                                                       \
    while (0)

#define LOGE(format, ...)                                                         \
    do {                                                                          \
        if (use_syslog) {                                                         \
            syslog(LOG_ERR, format, ## __VA_ARGS__);                              \
        } else {                                                                  \
            time_t now = time(NULL);                                              \
            char timestr[20];                                                     \
            strftime(timestr, 20, TIME_FORMAT, localtime(&now));                  \
            if (use_tty) {                                                        \
                fprintf(stderr, "\e[01;35m %s %s %d ERROR: \e[0m" format "\n", timestr,__FILE__, __LINE__, \
                        ## __VA_ARGS__);                                          \
            } else {                                                              \
                fprintf(stderr, " %s ERROR: " format "\n", timestr,               \
                        ## __VA_ARGS__);                                          \
            }                                                                     \
        }                                                                 \
    }                                                                       \
    while (0)

void ERROR(const char *s);
char *ss_itoa(int i);
int run_as(const char *user);
void FATAL(const char *msg);
void usage(void);
void daemonize(const char *path);
char *ss_strndup(const char *s, size_t n);
#ifdef HAVE_SETRLIMIT
int set_nofile(int nofile);
#endif

void *ss_malloc(size_t size);
void *ss_realloc(void *ptr, size_t new_size);

#define ss_free(ptr)     \
    do {                 \
        free(ptr);       \
        ptr = NULL;      \
    } while (0)

#endif // _UTILS_H
