#include <netinet/in.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <string.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>

#include "shadowfog_log.h"
#include "shadowfog_check.h"

#define CURRENT_FILE __FILE__
#define PEAR_CHECK_PORT 12304
#define PEAR_CHECK_WAIT_SEC 3

static int shadowfog_check_connect(char *extern_ip)
{
    int n;
    int fd;
    int flags;
    int error = 0;
    socklen_t len;
    fd_set rset, wset;
    struct timeval tval;
    struct sockaddr_in address;

    memset(&address, 0, sizeof(address));

    address.sin_family = AF_INET;
    address.sin_port   = htons(PEAR_CHECK_PORT);

    inet_pton(AF_INET, extern_ip, &address.sin_addr);

    fd = socket(AF_INET, SOCK_STREAM, 0);
    if (fd < 0)
    {
        LOGE("[%s] Function socket() ERROR!\n", CURRENT_FILE);
        return -1;
    }

    flags = fcntl(fd, F_GETFL, 0);
    fcntl(fd, F_SETFL, flags | O_NONBLOCK);

    if ( (n = connect(fd, (struct sockaddr *)&address, sizeof(address))) < 0)
    {
        if (errno != EINPROGRESS)
        {
            LOGE("[%s] Function connect ERROR!\n", CURRENT_FILE);
            return -1;
        }
    }

    if (n == 0)
    {
        LOGI("[sf-check] connected");
        close(fd);
        return 0;
    }

    FD_ZERO(&rset);
    FD_SET(fd, &rset);
    wset = rset;
    tval.tv_sec = PEAR_CHECK_WAIT_SEC;
    tval.tv_usec = 0;

    if ( (n = select(fd + 1, &rset, &wset, NULL, &tval)) <= 0)
    {
        close(fd);
        LOGI("[sf-check] connected timeout");
        return -1;
    }

    error = 0;
    if (FD_ISSET(fd, &rset) || FD_ISSET(fd, &wset))
    {
        len = sizeof(error);
        if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
        {
            close(fd);
            return -1;
        }

        if (error)
        {
            close(fd);
            return -1;
        }

        LOGI("[sf-check] Connected After Select");
        close(fd);
        return 0;
    }

    LOGI("[sf-check] Connected After Select NOT IN SET");
    return -1;
}

static int shadowfog_check_listen()
{
    int listen_fd;
    struct sockaddr_in address;

    memset(&address, 0, sizeof(address));

    address.sin_family = AF_INET;
    address.sin_port   = htons(PEAR_CHECK_PORT);
    address.sin_addr.s_addr = INADDR_ANY;

    listen_fd = socket(AF_INET, SOCK_STREAM, 0);
    if (listen_fd < 0)
    {
        LOGE("[%s] Function socket() ERROR!\n", CURRENT_FILE);
    }

    if (bind(listen_fd, (struct sockaddr *)&address, sizeof(address)) < 0)
    {
        LOGE("[%s] Function bind() ERROR!\n", CURRENT_FILE);
    }

    if (listen(listen_fd, 10) < 0)
    {
        LOGE("[%s] Function listen() ERROR!\n", CURRENT_FILE);
    }

    return listen_fd;
}

int shadowfog_check_nat(char *extern_ip)
{
    int i = 0;
    int j = 0;

    i = shadowfog_check_listen();
    j = shadowfog_check_connect(extern_ip);

    if (i > 0)
    {
        close(i);
    }

    return j;
}

