/*
 * 1. product
 * 2. destroy
 * 3. restart
 */
#include <ev.h>
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <libcork/core.h>
#include <libcork/ds.h>

#include "../utils.h"
#include "shadowfog_child_monitor.h"

#define MAX_LEN 512
#define MAX_ARGS 64
#define MAX_ARG_LEN 512

static struct cork_hash_table *shadowfog_hash_pid;

typedef struct shadowfog_child_s shadowfog_child_t;

struct shadowfog_child_s {
    int  pid;
    char command[MAX_LEN];
    char *argv[MAX_ARGS];
};

static void shadowfog_signal_child_cb(int signo);

void shadowfog_child_free_value(void *value)
{
    if (value == NULL)
    {
        return;
    }
    shadowfog_child_t *child = (shadowfog_child_t *)value;
    char **argvp = child->argv;
    for (; *argvp != NULL; argvp++)
    {
        free(*argvp);
    }
    free(child);
}

void shadowfog_child_init()
{
    if (signal(SIGCHLD, shadowfog_signal_child_cb) < 0)
    {
        //LOGI("signal error");
        return;
    }

    shadowfog_hash_pid  = cork_hash_table_new(0, 0);
    cork_hash_table_set_free_value(shadowfog_hash_pid, shadowfog_child_free_value);
}

void shadowfog_arg_cp(char *child_argv[], char *argv[])
{
    char *temp = NULL;
    for (; **argv != 0; argv++, child_argv++)
    {
        temp = malloc(64 * sizeof(char));
        strcpy(temp, *argv);
        *child_argv = temp;
    }

    *child_argv = NULL;
    printf("end\n");
}

void shadowfog_print_argv(char *argv[])
{
    for (; *argv != NULL; argv++)
    {
        printf("%s\n", *argv);
    }
}

int shadowfog_child_fork(char *command, char *argv[])
{
    ////LOGI("shadowfog_child_fork start, command: %s", command);

    intptr_t pid = -1;
    bool new = false;

    shadowfog_child_t *shadowfog_child = (shadowfog_child_t *)malloc(sizeof(shadowfog_child_t));
    memset(shadowfog_child, 0, sizeof(shadowfog_child_t));
    shadowfog_arg_cp(shadowfog_child->argv, argv);
    pid = fork();

    if (pid == -1)
    {
        //LOGI("[shadowfog-child] fork error!");
        return -1;
    }
    else if (pid > 0)
    {
        //LOGI("shadowfog_child_fork parent start, child pid: %d", (int)pid);

        shadowfog_child->pid  = pid;
        strcpy(shadowfog_child->command, command);

        cork_hash_table_put(shadowfog_hash_pid, (void *)pid, (void *)shadowfog_child, &new, NULL, NULL);
        //LOGI("shadowfog_child_fork parent end");

        return pid;
    }
    else if (pid == 0)
    {
        //LOGI("shadowfog_child_fork child start: command: %s, pid: %d", command, (int)getpid());

        //shadowfog_print_argv(shadowfog_child->argv);

        if (execvp(command, shadowfog_child->argv) < 0)
        {
            //LOGI("[child] execvp ERROR: errno: %d, %s", errno, strerror(errno));
        }

        exit(0);
    }

    return -1;
}

int shadowfog_child_refork(shadowfog_child_t *child)
{
/*
    pid changed
    how can i sent pid to server
    server get pid from pid_file
    new process save pid id pid_file
    !!
*/
    //LOGI("[child] hash_get: old pid: %d", child->pid);

    intptr_t pid = -1;
    bool new = false;

    pid = fork();

    if (pid == -1)
    {
        //LOGI("[shadowfog-child] fork error!");
        return -1;
    }
    else if (pid > 0)
    {
        //LOGI("[shadowfog-child] parent fork success!");
        child->pid  = pid;
        cork_hash_table_put(shadowfog_hash_pid, (void *)pid, (void *)child, &new, NULL, NULL);

        return pid;
    }
    else if (pid == 0)
    {
        //LOGI("[shadowfog-child] child fork success!");
        execvp(child->command, (char **)child->argv);
        //LOGI("[child] execvp ERROR");
        exit(-1);
    }

    return -1;
}

int shadowfog_child_destroy(intptr_t pid)
{
    if (cork_hash_table_delete(shadowfog_hash_pid, (void *)pid, NULL, NULL) < 0)
    {
        //LOGI("[child] UNKNOW PROCESS");
        return -1;
    }

    //LOGI("[child] child destroy");
    kill(pid, SIGINT);

    return 0;
}

static void shadowfog_signal_child_cb(int signo)
{
    //LOGI("[child] catched child exit: %d", signo);

    int  status;
    intptr_t pid = -1;
    
    void *value = NULL;

    /* avoid malloc in sigalarm callback function*/
    sigset_t block;
    sigemptyset(&block);
    sigaddset(&block, SIGALRM);
    sigprocmask(SIG_BLOCK, &block, NULL);

    while ( (pid = waitpid(-1, &status, WNOHANG)) > 0)
    {
        //LOGI("[child] wait pid success");
        value = cork_hash_table_get(shadowfog_hash_pid, (void *)pid);

        if (value != NULL)
        {
              cork_hash_table_put(shadowfog_hash_pid, (void *)pid, (void *)NULL, NULL, NULL, NULL);

              if (cork_hash_table_delete(shadowfog_hash_pid, (void *)pid, NULL, NULL) < 0)
              {
                  //LOGI("[child] UNKNOW PROCESS");
                  return;
              }

            shadowfog_child_refork((shadowfog_child_t *)value);
            //LOGI("[child] refork, status: %d!", status);
        }
        else
        {
            //LOGI("[child] catched child destory!");
        }
    }

    if (pid < 0)
    {
        //LOGI("[child] waitpid ERROR: %s", strerror(errno));
    }

    if (pid == 0)
    {
        //LOGI("[child] waitpid NOWAIT");
    }

    sigprocmask(SIG_UNBLOCK, &block, NULL);

}
