#include <curl/curl.h>
#include <string.h>

//#include "utils.h"
#include "shadowfog_log.h"
#include "shadowfog_curl.h"
#include "errno.h"
static size_t recved_len = 0;

/*
 * libcurl callback function
 */
static size_t shadowfog_curl_writecb(void *buf, size_t size, size_t nmemb, void *recv)
{
    size_t  recvbytes;

    // printf("size : %ld, nmemb: %ld,content:%s\n", size, nmemb, (unsigned char *)buf);
//	LOGI("[shadowfog-curl] size: %ld, nmemb: %ld, buf: %s\n\n\n\n\n", (long)size, (long)nmemb, (char *)buf);
    memcpy(recv + recved_len, buf, size * nmemb);
//	LOGI("[shadowfog-curl] size: %ld, nmemb: %ld, recv: %s\n\n\n\n\n", (long)size, (long)nmemb, (char *)recv);
    recvbytes = size * nmemb;
    recved_len += recvbytes;
    return recvbytes;
}


/*
 * execute http cmd with libcurl 
 *
 */
int shadowfog_curl_cmd(char *method, char *url,  char *par, char *header[], char *body, long body_size, char *recvbuf)
{
    CURLcode result;

    recved_len = 0;

    curl_global_init(CURL_GLOBAL_ALL);

    CURL *curl_handle = curl_easy_init();
    if (curl_handle == NULL)
    {
        LOGE("[curl] curl_easy_init error");
        return -1;
    }


    /*
     * For DEBUG
     */
    /*
    curl_easy_setopt(curl_handle, CURLOPT_DEBUGFUNCTION, shadowfog_curl_trace);
    curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1L);
     */


    /* 
     * Add header
     */

    struct curl_slist *curl_headers = NULL;
    
    if (header != NULL)
    {

        for (; *header; header++)
        {
            curl_headers = curl_slist_append(curl_headers, *header);
        }
        curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, curl_headers);
    }

    curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, curl_headers);


    /*
     * Set callback function
     *
     */

    curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, shadowfog_curl_writecb);
    curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, recvbuf);
    curl_easy_setopt(curl_handle, CURLOPT_ERRORBUFFER, NULL);


    /*
     * POST and GET, decide if there is body data
     *
     */

    if (strcmp(method, "POST") == 0)
    {

        curl_easy_setopt(curl_handle, CURLOPT_POST, 1L);

        curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDSIZE, body_size);

        curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, body);

    }
    else if (strcmp(method, "GET") == 0)
    {

    }
    else
    {
        return -1;
    }


    /* other options */
    curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL,  1);
    curl_easy_setopt(curl_handle, CURLOPT_URL,       url);
    curl_easy_setopt(curl_handle, CURLOPT_NOBODY,    0L);
    curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT,   10);
    curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36");
    curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
    curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);


    /* NO DNS CACHE ;  NO HANDLE REUSE*/
    curl_easy_setopt(curl_handle, CURLOPT_FORBID_REUSE,   1L);
    curl_easy_setopt(curl_handle, CURLOPT_FRESH_CONNECT,  1L);
    curl_easy_setopt(curl_handle, CURLOPT_DNS_USE_GLOBAL_CACHE, 0L);


    /* perform the url */
    result = curl_easy_perform(curl_handle);
    if (result != CURLE_OK)
    {
        LOGI("[%s] Curl ERROR: %d = %s; errno:%d", LOG_TITLE, result, curl_easy_strerror(result), errno);

        curl_slist_free_all(curl_headers);
        curl_easy_cleanup(curl_handle);
        curl_global_cleanup();

        return -1;
    }

    double size;
    long ret;
    curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &ret);
    LOGI("[%s] Curl response code: %ld", LOG_TITLE, ret);
    curl_easy_getinfo(curl_handle, CURLINFO_SIZE_DOWNLOAD, &size);

    // clean up
    //
    curl_slist_free_all(curl_headers);
    curl_easy_cleanup(curl_handle);
    curl_global_cleanup();

    //return retcode;
    return (int)size;
}
