#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/ioctl.h>
#include <net/if.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#include <sys/sysinfo.h>
#include <sys/utsname.h>
#include <string.h>

#include "shadowfog_log.h"
#include "shadowfog_curl.h"
#include "shadowfog_json.h"
#include "shadowfog_host.h"
#include "shadowfog_base64.h"
#include "shadowfog_mapping.h"
#include "shadowfog_fog_info.h"

shadowfog_fog_info_t shadowfog_fog_info;

static char *shadowfog_curl_header[] = {
    "Content-Type: application/json",
    "Cache-Control: no-chache",
    NULL
};


static char *shadowfog_fog_info_host_address[] = {
    "https://git.oschina.net/ShadowFogNetwork/Information/raw/master/ss-manager.json",
    "https://host.ablive.net/host.json",
    NULL
};


static int shadowfog_get_net_type(char *host, enum NET_TYPE *type, char *mac, char *extern_ip);

/*
 * Get Processor Serial Numbers
 * Disable it , cause mips platform not support this
 */

void shadowfog_get_PSN(char *PSN)
{
    sprintf(PSN, "shadowfog");
}

int shadowfog_get_CpuNum()
{
    return sysconf( _SC_NPROCESSORS_ONLN );
}


/*
 * Detect system memory size
 */
int shadowfog_detect_memsize(char * memory)
{
    struct sysinfo si;

    /* Conversion constants. */
    const long minute = 60;
    const long hour   = minute * 60;
    const long day    = hour   * 24;
    const double megabyte = 1024 * 1024;

    /* Obtain system statistics. */
    sysinfo(&si);

    /* Summarize interesting values. */
    LOGI ("[sf-info] system uptime : %ld days, %ld:%02ld:%02ld\n", 
            si.uptime / day, (si.uptime % day) / hour, 
            (si.uptime % hour) / minute, si.uptime % minute);

    LOGI ("[sf-info] total RAM   : %5.1f MB\n", si.totalram / megabyte);
    LOGI ("[sf-info] free RAM   : %5.1f MB\n",  si.freeram / megabyte);
    LOGI ("[sf-info] process count : %d\n",     si.procs);
   
    //sprintf(memory,"%5.1f:%5.1f",si.freeram / megabyte,si.totalram / megabyte);
    sprintf(memory,"%d", (int)(si.freeram / megabyte));
    LOGI ("[sf-info] memory is  : %s\n", memory);

    return  0;
}

void shadowfog_detect_platform(char *platform)
{
    char   *os = NULL;
    struct utsname buffer;
    sprintf(platform, "%s", PLATFORM);
    return;
   
    if (uname(&buffer) != 0)
    {
        LOGI("[sf-info] uname error");
    }

    LOGI("[sf-info] system name = %s", buffer.sysname);
    LOGI("[sf-info] node name   = %s", buffer.nodename);
    LOGI("[sf-info] release     = %s", buffer.release);
    LOGI("[sf-info] version     = %s", buffer.version);
    LOGI("[sf-info] machine     = %s", buffer.machine);
   
    #ifdef _GNU_SOURCE
       LOGI("[sf-info] domain name = %s\n", buffer.domainname);
    #endif

    shadowfog_fog_info.hardware_platform = HARDWARE_PLATFORM_PEAR;

    if (uname(&buffer) != 0)
    {
        LOGI("[sf-info] uname error");
    }
    if(strstr(buffer.version, "Ubuntu") )  //  #78-Ubuntu SMP Fri Dec 9
    {
        os = "Ubuntu";
    }
    if(strstr(buffer.version, "OpenWrt") )  //  #78-Ubuntu SMP Fri Dec 9
    {
        os = "OpenWrt";
    }

    sprintf(platform,"%s-%s-%s", buffer.sysname, os, buffer.machine);

    LOGI("[sf-info] platform is %s ", shadowfog_fog_info.platform);

}

/*
 * Get mac address
 * http://stackoverflow.com/questions/1779715/how-to-get-mac-address-of-your-machine-using-a-c-program
 *
 */

int  shadowfog_get_mac(char * mac_address)
{
    struct ifreq  ifr;
    struct ifconf ifc;
    int  success = 0;
    char buf[1024]       = {0x00};
    unsigned char mac[6] = {0x00};

    int sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (sock == -1)
    {
        /* handle error*/ 
    };

    ifc.ifc_buf = buf;
    ifc.ifc_len = sizeof(buf);

    if (ioctl(sock, SIOCGIFCONF, &ifc) == -1)
    {
        /* handle error */
    }

    struct ifreq *it = ifc.ifc_req;
    const struct ifreq* const end = it + (ifc.ifc_len / sizeof(struct ifreq));

    for (; it != end; ++it)
    {
        strcpy(ifr.ifr_name, it->ifr_name);

        if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0)
        {
            if (! (ifr.ifr_flags & IFF_LOOPBACK))
            {
                // don't count loopback
                if (ioctl(sock, SIOCGIFHWADDR, &ifr) == 0)
                {
                    success = 1;
                    break;
                }
            }
        }
        else
        {
            /* handle error */
        }
    }

    if (success)
    {
        memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
        // sprintf(mac_address, "%C:%C:%C:%C:%C:%C", str[0], str[1], str[2], str[3], str[4], str[5]);
	    sprintf(mac_address, "%02x:%02x:%02x:%02x:%02x:%02x", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    }

    close(sock);

    return 0;
}

void shadowfog_get_external_ip(char *value)
{
    char *api_url[] = {
        "https://api.ipify.org?format=json",
        "http://ipinfo.io/json",
        "http://myexternalip.com/json"
    };

    int  i = 0;
    int  s = 0;
    char temp[12048] = {0x00};

    for (i = 0; i < 3; i++)
    {
        s = shadowfog_curl_cmd("GET", api_url[i], NULL, shadowfog_curl_header, NULL, 0, temp);

        if (shadowfog_json_get_string(temp, "ip", value) > 0)
        {
            return;
        }
    }

    LOGI("[sf-info] Can't get external ip!");
}

void shadowfog_detect_local_ip(char *buf, int buflen) 
{
	socklen_t len;
	struct sockaddr_in addr;

	int fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd == -1)
    {
		goto error;
    }

	addr.sin_port   = htons(1);
	addr.sin_family = AF_INET;
	inet_pton(AF_INET, "1.2.3.4", &addr.sin_addr.s_addr);

	if (connect(fd, (const struct sockaddr*) &addr, sizeof(addr)) < 0)
    {
		goto error;
    }

	len = sizeof(addr);
	if (getsockname(fd, (struct sockaddr*) &addr, &len) < 0)
    {
		goto error;
    }

	if (getnameinfo((const struct sockaddr*) &addr, sizeof(addr),
			buf, buflen, NULL, 0, NI_NUMERICHOST) != 0)
    {
		goto error;
    }

	close(fd);
	return;

error:
	if (fd != -1)
		close(fd);

	strcpy(buf, "127.0.0.1");
}

static int shadowfog_get_net_type (char * host, enum NET_TYPE *type, char *mac, char *extern_ip)
{
    int  s = 0;
    char cmd [512]    = {0x00};
    char temp[4096]   = {0x00};
    char net_type[10] = {0x00};

    sprintf(cmd ,"%s/fogs?mac=%s&ip=%s", host, mac, extern_ip);
    LOGI("[sf-info] Ping cmd: %s", cmd);
    
    s = shadowfog_curl_cmd("GET", cmd, NULL, shadowfog_curl_header, NULL, 0, temp);
    if (s < 0)
    {
        return -1;
    }

    LOGI("[sf-info] read feedback bytes %d , and content : %s", s, temp);

    /*
        return like this:
        {"code":"0","msg":"pong","net_type":"GI"}
    */


    shadowfog_json_get_string(temp, "net_type", net_type);

    if (!strcmp(net_type,"GI"))
    {
       *type = NET_TYPE_GI;
    }
    else if (!strcmp(net_type,"EDU"))
    {
       *type = NET_TYPE_EDU;
    }
    else if (!strcmp(net_type,"DC"))
    {
       *type = NET_TYPE_DC;
    }
    else
    {
       *type = NET_TYPE_GI;
    }
	
    if (strstr(temp,"pong"))
    {
        LOGI("[sf-info] ping get pong , and net_type is %d", *type);
        return 0;
    }

    return -1;
}

int shadowfog_fog_info_ipchange(shadowfog_fog_info_t *info)
{
    if (strcmp(info->extern_ip, info->old_extern_ip) == 0)
    {
        return 0;
    }
    else
    {
        strcpy(info->old_extern_ip, info->extern_ip);
        return 1;
    }
}

void shadowfog_fog_info_detect()
{
    LOGI("[sf-info] ==============shadowfog_fog_info_detect Starting===========");
    if (shadowfog_fog_info.max_server == 0)
    {
        shadowfog_fog_info.max_server = PEAR_MAX_SS_SERVER;
    }

    shadowfog_get_mac(shadowfog_fog_info.mac);
    LOGI("[sf-info] mac:%s\n", shadowfog_fog_info.mac);

    shadowfog_get_external_ip(shadowfog_fog_info.extern_ip);
    LOGI("[sf-info] extern_ip:%s\n", shadowfog_fog_info.extern_ip);
    shadowfog_detect_local_ip(shadowfog_fog_info.local_ip, sizeof(shadowfog_fog_info.local_ip));
    LOGI("[sf-info] local_ip:%s\n", shadowfog_fog_info.local_ip);

    if (strlen(shadowfog_fog_info.host_ip) < 2)
    {
        shadowfog_get_host(shadowfog_fog_info.host_ip, shadowfog_fog_info_host_address, 3, "shadowfog_net_type");
    	LOGI("[sf-info] host_address:%s\n", shadowfog_fog_info.host_ip);
    }

    if (shadowfog_fog_info_ipchange(&shadowfog_fog_info))
    {
        shadowfog_get_net_type(shadowfog_fog_info.host_ip, &shadowfog_fog_info.net_type, shadowfog_fog_info.mac, shadowfog_fog_info.extern_ip);
    	LOGI("[sf-info] host_address:%s\n", shadowfog_fog_info.host_ip);
    }

    shadowfog_get_PSN(shadowfog_fog_info.psn);
    shadowfog_detect_memsize(shadowfog_fog_info.memery);
    shadowfog_fog_info.cpu_number = shadowfog_get_CpuNum();
    shadowfog_detect_platform(shadowfog_fog_info.platform);

    LOGI("[sf-info] external ip is : %s and local ip is %s and mac is %s",
            shadowfog_fog_info.extern_ip,
            shadowfog_fog_info.local_ip,
            shadowfog_fog_info.mac);

    shadowfog_mapping_init(shadowfog_fog_info.extern_ip, shadowfog_fog_info.local_ip);

    shadowfog_fog_info.is_public_ip    = shadowfog_mapping_info.is_public_ip;
    shadowfog_fog_info.is_support_pmp  = shadowfog_mapping_info.is_support_pmp;
    shadowfog_fog_info.is_support_upnp = shadowfog_mapping_info.is_support_upnp;

    LOGI("[sf-info] ==============shadowfog_fog_info_detect END===========");
}

/*
 * Detect machine information.
 */
void shadowfog_fog_info_init()
{
    memset(&shadowfog_fog_info, 0, sizeof(shadowfog_fog_info_t));
    shadowfog_fog_info_detect();
}

