#include <stdio.h>
#include <string.h>
#include "jansson.h"
#include "shadowfog_log.h"
#include "shadowfog_jconf.h"

#define MAX_BUF 2048

static int options_bits = 0;

char *options[] = {
    "mtu",
    "mptcp",
    "nofile",
    "shadowfog_mode",
    "user",
    "method",
    "timeout",
    "pid_path",
    "password",
    "shadowfog_user",
    "list_ipset",
    "local_addr",
    "remote_dns",
    "redir_port",
    "tunnel_port",
    "shadowfog_passwd",
    "remote_port",
    "remote_host",
    "shadowfog_dnsurl",
    "domain_file",
    "shadowfog_dnsmasq",
    NULL
};

router_config_t *shadowfog_json_read_conf(char *path)
{
    int i = 0;
    json_t *obj;
    json_t *value;
    const char *key;
    json_error_t error;
    router_config_t *config = NULL;
    char conf_buf[MAX_BUF]  = {0x00};

    FILE *f = fopen(path, "r");
    if (f == NULL)
    {
        LOGI("[jconf] can't open configure file: %s", path);
        return NULL;
    }

    fread(conf_buf, sizeof(char), MAX_BUF, f);
    fclose(f);

    obj = json_loads(conf_buf, 0, &error);
    if (obj == NULL)
    {
        LOGI("[ss-json] json_loads error!! json_string:%s", conf_buf);
        LOGI("[ss-json] json_error_code: %s", error.text);
    }

    config = (router_config_t *)malloc(sizeof(router_config_t));
    memset(config, 0x00, sizeof(router_config_t));

    json_object_foreach(obj, key, value)
    {

        if (!json_is_string(value))
        {
            LOGI("config file error");
            return NULL;
        }

        for (i = 0; options[i] != NULL; i++)
        {
            if ((1 << i) & options_bits)
            {
                continue;
            }

            if (strcmp(key, options[i]) == 0)
            {
                void *tt;
                char **temp;
                options_bits |= (1 << i);
                LOGI("[jconf] key: %s, value: %s", options[i], json_string_value(value));
                tt = (void *)config + i * sizeof(void *);
                temp  = (char **)tt;
                *temp = (char *)malloc(512 * sizeof(char));
                strcpy(*temp, json_string_value(value));
            }
        }
    }
    LOGI("jconf end");

    json_decref(obj);
    return config;
}
