#include <string.h>
//#include "utils.h"
#include "shadowfog_log.h"
#include "jansson.h"
#include "shadowfog_json.h"


int shadowfog_json_foreach_object(json_t *obj, const char *key, char *value)
{
    LOGI("[shadowfog-router] shadowfog_json_foreach_object start");
    json_t *_value;
    json_t *temp_obj;
    const char *_key;

    json_object_foreach(obj, _key, _value)
    {
        temp_obj = json_object_get(obj, _key);

        if (json_is_object(temp_obj))
        {
            LOGI("[shadowfog-router] is object");
            if (shadowfog_json_foreach_object(temp_obj, key, value) > 0)
            {
                return 1;
            }
        }
        else if (json_is_array(temp_obj))
        {
            LOGI("[shadowfog-router] is array");
            if (shadowfog_json_foreach_array(temp_obj, key, value) > 0)
            {
                return 1;
            }
        }
        else if (json_is_string(temp_obj))
        {
            LOGI("[shadowfog-router] is string");
            if (strcmp(key, _key) == 0)
            {
                LOGI("[shadowfog-router] find it");
                strcpy(value, json_string_value(_value));
                LOGI("[shadowfog-router] copy it");
                return 1;
            }
        }
        else
        {
            LOGI("[shadowfog-router] unknow type");
        }
    }

    return -1;
}

int shadowfog_json_foreach_array(json_t *obj, const char *key, char *value)
{
    LOGI("shadowfog_json_foreach_array start");
    size_t index     = 0;
    json_t *_value   = NULL;
    json_t *temp_obj = NULL;
    const char *_key = NULL;

    json_array_foreach(obj, index, _value)
    {
        temp_obj = json_array_get(obj, index);

        if (json_is_object(temp_obj))
        {
            if (shadowfog_json_foreach_object(temp_obj, key, value) > 0)
            {
                return 1;
            }
        }
        else if (json_is_array(obj))
        {
            if (shadowfog_json_foreach_array(temp_obj, key, value) > 0)
            {
                return 1;
            }
        }
        else if (json_is_string(temp_obj))
        {
            if (strcmp(key, _key) == 0)
            {
                LOGI("[shadowfog-router] array find it");
                strcpy(value, json_string_value(_value));
                LOGI("[shadowfog-router] array copy it");
                return 1;
            }
        }
    }

    return -1;
}

int shadowfog_json_get_string(char *buf, char *key, char *myvalue)
{
    LOGI("shadowfog_json_get_string start");
    json_error_t error;
    json_t *obj = NULL;

    obj = json_loads(buf, 0, &error);
    if (obj == NULL)
    {
        LOGI("[ss-json] json_loads error!! json_string:%s", buf);
        LOGI("[ss-json] json_error_code: %s", error.text);
    }

    if (shadowfog_json_foreach_object(obj, key, myvalue) > 0)
    {
        json_decref(obj);
        return 1;
    }
    else
    {
        LOGI("[shadowfog-router] Invalid config string");
    }

    json_decref(obj);

    return -1;
}

void shadowfog_json_get_server(char *buf, char *server, char *server_port, char *password)
{
    LOGI("shadowfog_json__get_server start");
    json_error_t error;
    json_t *obj = NULL;

    obj = json_loads(buf, 0, &error);
    if (!shadowfog_json_foreach_object(obj, "server", server))
    {
        LOGI("[shadowfog-router] Invalid config string");
    }

    if (!shadowfog_json_foreach_object(obj, "server_port", server_port))
    {
        LOGI("[shadowfog-router] Invalid config string");
    }

    if (!shadowfog_json_foreach_object(obj, "password", password))
    {
        LOGI("[shadowfog-router] Invalid config string");
    }

    json_decref(obj);
}

int shadowfog_json_get_string_file(char *path, char *key, char *myvalue)
{
    LOGI("shadowfog_json_get_string start");
    json_error_t error;
    json_t *obj = NULL;

    obj = json_load_file(path, 0, &error);
    if (obj == NULL)
    {
        LOGI("[ss-json] json_loads error!! json_file:%s", path);
        LOGI("[ss-json] json_error_code: %s", error.text);
    }

    if (shadowfog_json_foreach_object(obj, key, myvalue) > 0)
    {
        json_decref(obj);
        return 1;
    }
    else
    {
        LOGI("[shadowfog-router] Invalid config string");
    }

    json_decref(obj);

    return -1;
}
