#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <locale.h>
#include <signal.h>
#include <string.h>
#include <strings.h>
#include <time.h>
#include <unistd.h>
#include <getopt.h>
#include <math.h>
#include <ctype.h>
#include <limits.h>
#include <dirent.h>

#ifndef __MINGW32__
#include <netdb.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <pthread.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <pwd.h>
#endif

#include <libcork/core.h>

#ifdef __MINGW32__
#include "win32.h"
#endif

#if defined(HAVE_SYS_IOCTL_H) && defined(HAVE_NET_IF_H) && defined(__linux__)
#include <net/if.h>
#include <sys/ioctl.h>
#define SET_INTERFACE
#endif

#include "manager.h"
#include "shadowfog_log.h"
#include "shadowfog_host.h"
#include "shadowfog_json.h"
#include "shadowfog_mapping.h"
#include "shadowfog_version.h"
#include "shadowfog_fog_info.h"

#include "shadowfog_port.h"
#include "shadowfog_curl.h"
#include "shadowfog_utils.h"
#include "shadowfog_speed.h"
#include "shadowfog_base64.h"
#include "shadowfog_passwd.h"
#include "shadowfog_common.h"
#include "shadowfog_manager.h"
#include "shadowfog_child_monitor.h"

#define CURRENT_FILE __FILE__


static int  server_error_number = 0;


void add_server(struct manager_ctx *manager, struct server *server);
void kill_server(char *prefix, char *pid_file);
void stop_server(char *prefix, char *port);
void remove_server(char *prefix, char *port);

extern struct manager_ctx manager;
extern int verbose;

extern  struct cork_hash_table *server_table;

char shadowfog_real_host_address[1024];

#ifndef BUF_SIZE
#define BUF_SIZE 65535
#endif

// another callback, this time for a time-out
#if 0
 -t,  --tries=NUMBER              set number of retries to NUMBER (0 unlimits)
       --retry-connrefused         retry even if connection is refused
 -O,  --output-document=FILE      write documents to FILE

--no-cache                  disallow server-cached data
#endif 

#define MAX_STRING 1024
#define MAX_PASS   512

typedef struct {
    char port[MAX_PASS];
    char passwd[MAX_PASS];
} shadowfog_port_passwd_t;

typedef struct {
    char prs_buf[MAX_STRING];
    int  prs_num;
    shadowfog_port_passwd_t prs_port_passwd[50];
} shadowfog_schedule_t;

shadowfog_schedule_t  shadowfog_schedule;

float node_coeficient   = 1;
float normal_coeficient = 0.5;


/*
 * keep channel info after restarting
 *
 */
#define MAX_CHANNEL_LEN 16

typedef struct keep_channel_s keep_channel_t;

struct keep_channel_s {
    int  pid;
    char port[MAX_CHANNEL_LEN];
    char password[MAX_CHANNEL_LEN];
    keep_channel_t *next;
};

keep_channel_t *keep_channel_list = NULL;
int keep_channel_list_num = 0;

void add_keep_channel_list(char *conf_file, int pid)
{
    keep_channel_t *temp = NULL;

    temp = malloc(sizeof(keep_channel_t));

    if (shadowfog_json_get_string_file(conf_file, "server_port", temp->port) < 0)
    {
        LOGI("[sf-manager] get server_port error");
        free(temp);
        return;
    }

    if (shadowfog_json_get_string_file(conf_file, "password", temp->password) < 0)
    {
        LOGI("[sf-manager] get password error");
        free(temp);
        return;
    }

    temp->pid = pid;

    LOGI("[sf-manager] port:%s password:%s pid: %d", temp->port, temp->password, temp->pid);

    temp->next = keep_channel_list;
    keep_channel_list = temp;

    keep_channel_list_num++;
}

void shadowfog_set_max_server()
{
    int min      = 0;
    int upload   = 0;
    int download = 0;

    upload   = atoi(shadowfog_speed_info.upload_max);
    download = atoi(shadowfog_speed_info.download_max);

    min = upload < download ? upload : download;

    shadowfog_fog_info.max_server = min * normal_coeficient * node_coeficient + 1;
    if (shadowfog_fog_info.max_server < 25)
    {
        shadowfog_fog_info.max_server = 25;
    }

    LOGI("[ss-manager] upload:%d, download:%d, min:%d, max_server:%d", upload, download, min, shadowfog_fog_info.max_server);
}


int shadowfog_balance_max_channel()
{
    int r   = 0;
    int max = 0;
    struct cork_hash_table_entry   *entry;
    struct cork_hash_table_iterator server_iter;
  
    cork_hash_table_iterator_init(server_table, &server_iter);

    while ((entry = cork_hash_table_iterator_next(&server_iter)) != NULL)
    {
        r = r + 1;
    }

    LOGI("[shadowfog bug001] shadowfog_balance_max_channel r is %d, and max is %d, keep_channel_list_num: %d", r, shadowfog_fog_info.max_server, keep_channel_list_num);

    /*
     * Copy  log from server :  [shadowfog bug001] shadowfog_balance_max_channel r is 0 and max is 2
     * DEBUG for upnp :  shadowfog_balance_max_channel r is 31 and max is 28
     * DEBUG shadowfog_balance_max_channel r is 31 and max is 0
     */

    max = shadowfog_fog_info.max_server > keep_channel_list_num ? shadowfog_fog_info.max_server : keep_channel_list_num;

    if (r <= max)
    {
        shadowfog_random_run_servers(max - r);
    }

    return max -r;
    return shadowfog_fog_info.max_server - r;
}


#include <time.h>
#include <stdlib.h>
#include "havege.h"


// get from man rand
static unsigned long next = 1;

/* RAND_MAX assumed to be 32767 */
int myrand(void) {
   next = next * 1103515245 + 12345;
   return((unsigned)(next/65536) % 32768);
}

void mysrand(unsigned int seed) {
   next = seed;
}

char shadowfog_special[]    = {'#','@','$','&','*','#','@','$','*','&','?'};
char shadowfog_lower_case[] = {'a','b','c','d','e','f','g','h','i','j','k'};
char shadowfog_upper_case[] = {'L','M','N','O','P','Q','R','S','T','U','V'};

static int
shadowfog_random_run_server()
{

    int  r = 0;
    int  freeport;
    char port[256];
    char password[256];
    keep_channel_t *temp = NULL;

    if (keep_channel_list != NULL)
    {
        LOGI("===============keep_channel_list NOT NULL");
        temp = keep_channel_list;
        keep_channel_list = temp->next;

        if (temp->pid > 0)
        {
            kill(temp->pid, SIGTERM);
            LOGI("[sf-manager] kill pid: %d", temp->pid);
        }

        strcpy(port, temp->port);
        strcpy(password, temp->password);
        free(temp);
        keep_channel_list_num--;
    }
    else
    {
        freeport = get_random_port(shadowfog_fog_info.net_type);

        LOGI("[shadowfog] get port : %d ", freeport);

        shadow_gen_passwd (password,0);

        sprintf(port,"%d",freeport);  
    }

    r = shadowfog_mapping_port_alloc(port);
    if (r)
    {
        return -1;
    }

    struct server *server = (struct server *)malloc(sizeof(struct server));
    strncpy(server->port, port, 8);
    strncpy(server->password, password, 128);

    server->traffic    = 0;
    server->traffic_tx = 0;
    server->traffic_rx = 0;
    server->traffic_minute = 0;

    LOGI("[shadowfog] port : %10s & password : %s ",port,server->password);
    remove_server(working_dir, server->port);
    add_server(&manager, server);

    return 0;
}

void shadowfog_random_run_servers(int number)
{
    int i = 0;

    for (i = 0; i < number; i++)
    {
        LOGI("[shadowfog_manager] max server is %d shadowfog_random_run_servers i is %d and number is %d", shadowfog_fog_info.max_server, i, number);

        if(shadowfog_random_run_server())
        {
            LOGI("run server error , may be upnp or pmp limit alloc number");
		    break;
        }
        /* Sleep for What ? Check port use or not ! But run_server not use the port*/
        sleep(1);
    }
    LOGI("[shadowfog_manager] shadowfog_random_run_servers end");

    return;
}

void shadowfog_send_delete_cmd (char * curl , char * host , char * method)
{
    int  s = 0;
    char cmd[512]   = {0x00};
    char post[512]  = {0x00};
    char temp[4096] = {0x00};

    sprintf(post, "{\"auth\":\"%s\","
                  "\"mac\":\"%s\","
                  "\"local_ip:\"\"%s\","
                  "\"ip\":\"%s\"}",
                  "shadowfog",
                  shadowfog_fog_info.mac,
                  shadowfog_fog_info.local_ip,
                  shadowfog_fog_info.extern_ip);

    LOGI("[shadowfog-manager] post content: %s", post);

    sprintf(cmd ,"%s/fogs/channels", host);
    LOGI("[shadowfog] cmd: %s", cmd);

    s = shadowfog_curl_cmd("POST", cmd, NULL, shadowfog_curl_header, post, (long)strlen(post), temp);

    LOGI("read feedback bytes %d , and content : %s", s, temp);
    return;
	
}

// :{"Ip":"0.0.0.0","Fogs":[{"Port":"8001","Traffic":"20000"},{"Port":"8002","Traffic":"30000"}]}
//  //curl -H "Content-Type: application/json" -X POST -d '{"Fogs":[{"Port":"8001","Traffic":"20000"},
//  {"Port":"8002","Traffic":"30000"}],"Ip":"0.0.0.0"}' http://localhost:8080/fogs

// w+     Open  for  reading  and writing.  The file is created if it does
//           not exist, otherwise it is truncated.  The stream is  positioned
//           at the beginning of the file.

//-X, --request <command>
//             (HTTP) Specifies a custom request method to use when communicating  with the HTTP server.
void  

shadowfog_dump_stat (char * host)
{
    int s = 0;
    int server_count = 0;

    char *cmd  = NULL;
    char *temp = NULL;
    char *post = NULL;

    struct cork_hash_table_entry   *entry = NULL;
    struct cork_hash_table_iterator server_iter;

    cmd  = (char *) malloc(1024  * sizeof(char));
    temp = (char *) malloc(5190  * sizeof(char));
    post = (char *) malloc(61440 * sizeof(char));

    shadowfog_fog_info_detect();

    cork_hash_table_iterator_init(server_table, &server_iter);

    sprintf(post, "{"
                  "\"ip\":\"%s\","
                  "\"local_ip:\":\"%s\","
                  "\"auth\":\"%s\","

                  "\"mac\":\"%s\","
                  "\"psn\":\"%s\","
                  "\"cpu\":\"%d\","
                  "\"memory\":\"%s\","
                  "\"upnp\":\"%d\","
                  "\"pmp\":\"%d\","

                  "\"ss_server\":\"%d\","
                  "\"speed_update\":\"%d\","
                  "\"ping\":\"%s\","
                  "\"download\":\"%s\","
                  "\"upload\":\"%s\","
                  "\"rate_tx_02\":%f,"
                  "\"rate_tx_02_max\":%f,"
                  "\"rate_tx_10\":%f,"
                  "\"rate_tx_10_max\":%f,"
                  "\"rate_tx_40\":%f,"
                  "\"rate_tx_40_max\":%f,"
                  "\"rate_rx_02\":%f,"
                  "\"rate_rx_02_max\":%f,"
                  "\"rate_rx_10\":%f,"
                  "\"rate_rx_10_max\":%f,"
                  "\"rate_rx_40\":%f,"
                  "\"rate_rx_40_max\":%f,"
                  "\"version\":\"%s\","
                  "\"shellresult\":\"%s\","
                  "\"channels\":[",
                  shadowfog_fog_info.extern_ip,
                  shadowfog_fog_info.local_ip,
                  "shadowfog",

                  shadowfog_fog_info.mac,
                  shadowfog_fog_info.psn,
                  shadowfog_fog_info.cpu_number,
                  shadowfog_fog_info.memery,
                  shadowfog_mapping_info.upnp,
                  shadowfog_mapping_info.pmp,

                  shadowfog_fog_info.max_server,
                  shadowfog_speed_info.speed_update,
                  shadowfog_speed_info.ping,
                  shadowfog_speed_info.download,
                  shadowfog_speed_info.upload,
                  rate_tx->rate_02.rate,
                  rate_tx->rate_02.rate_max,
                  rate_tx->rate_10.rate,
                  rate_tx->rate_10.rate_max,
                  rate_tx->rate_40.rate,
                  rate_rx->rate_02.rate,
                  rate_rx->rate_02.rate_max,
                  rate_rx->rate_10.rate,
                  rate_rx->rate_10.rate_max,
                  rate_rx->rate_40.rate,
                  rate_rx->rate_40.rate_max,
                  rate_tx->rate_40.rate_max,
                  SHADOWFOG_VERSION_STR,
                  shadowfog_fog_info.shellresult);

    while ((entry = cork_hash_table_iterator_next(&server_iter)) != NULL) {
        struct server *server = (struct server *)entry->value;

        LOGI("[shadowfog] %d dump traffic %s : %" PRIu64 "  %" PRIu64 "   %" PRIu64 "",server_count++, server->port, server->traffic , server->traffic_tx ,server->traffic_rx);

        sprintf(post + strlen(post),
                "{\"port\":\"%s\","
                " \"password\":\"%s\" ,"
                "\"traffic\":\"%" PRIu64 "\","
                "\"traffic_tx\":\"%" PRIu64 "\","
                "\"traffic_rx\":\"%" PRIu64 "\","
                "\"traffic_minute\":\"%" PRIu64 "\"},",
                server->port,
                server->password,
                server->traffic,
                server->traffic_tx,
                server->traffic_rx,
                server->traffic_minute);

    }

    cork_hash_table_iterator_init(server_table, &server_iter);

    sprintf(post + strlen(post) - 1, "]}");
    LOGI("[%s] POST: %s\n", LOG_TITLE, post);
    sprintf(cmd ,"%s/fogs/channels", host);
    int cmd_length = strlen(cmd);
    LOGI("[%s] cmd: %s and length %d", LOG_TITLE, cmd, cmd_length);

    s = shadowfog_curl_cmd("POST", cmd, NULL, shadowfog_curl_header, post, (long)strlen(post), temp);
    printf("[s] : %d\n", s);
    if (s > 0)
    {
        strcpy(shadowfog_schedule.prs_buf, temp);
        LOGI("[%s] Read feedback bytes %d , and content : %s", LOG_TITLE, s, temp);
    }

    return;

}


// 0 : means valid reponse
int shadowfog_is_delete_channel_valid()
{
    if (!strstr(shadowfog_schedule.prs_buf, "port_password"))
    {
        return -1; 
    }

    return 0;
}

void shadowfog_parse_shell(char * shell)
{
    char cmd[256]={0x00};
    shadowfog_json_get_string(shell, "shell", cmd);
    //LOGI("shadowfog_parse_shell shell is %s and cmd  is  %s", shell , cmd);
    char temp[4096]={0x00};
    char base64_temp[4096]={0x00};
    FILE * pipe_reader;
    pipe_reader = popen(cmd, "r");
    int i = fread(temp, 1, 4096, pipe_reader);
    // added by yongming.li for server crash on fread
    pclose(pipe_reader);
    if (i==0)
    {
        return;
    }
   
    Base64encode(base64_temp,temp,strlen(temp));
    //LOGI("shell result base64_temp length    %d" , (int)strlen(base64_temp));
    strcpy(shadowfog_fog_info.shellresult, base64_temp);
}

void shadowfog_parse_config(char * config)
{
     char speed_test[256]={0x00};
    shadowfog_json_get_string(config, "speed_test", speed_test);
     //LOGI("shadowfog_parse_config is  %s and speed_test is %s", config,speed_test);
     if(!strcmp(speed_test,"0"))
     {
          //LOGI("[shadowfog] now disable speed test");
          shadowfog_speed_info.speed_enable=0;
     }
     else
     {
         shadowfog_speed_info.speed_enable=1;
     }
}

void shadowfog_parse_server()
{
    char fog_shell[1024]   = {0x00};
    char fog_config[1024]  = {0x00};
    char base64_temp[4096] = {0x00};
	
    if (shadowfog_is_delete_channel_valid())
    {
    	 return ;
    }

/*
    { "port_password":{  },"Fog_config":"ewogICAgInNwZWVkX3Rlc3QiOiAiMCIKfQo=","Fog_shell":"ewogICAgInNoZWxsIjogImlmY29uZmlnIgp9Cg==" }
*/

    shadowfog_json_get_string(shadowfog_schedule.prs_buf, "fog_config", fog_config);
    shadowfog_json_get_string(shadowfog_schedule.prs_buf, "fog_shell", fog_shell);

    if (strlen(fog_config))
    {
        Base64decode(base64_temp, fog_config);
        shadowfog_parse_config(base64_temp);
    }

    memset(base64_temp,0x00,4096);

    if (strlen(fog_shell))
    {
        Base64decode(base64_temp, fog_shell);
        shadowfog_parse_shell(base64_temp);
    }
}

int shadowfog_remove_server_from_scheduler()
{
    int  i   = 0;
    int  ret = 0;

    LOGI("[%s] shadowfog_remove_server_from_scheduler", LOG_TITLE);

    if (shadowfog_is_delete_channel_valid())
    {
        // do nothing for schduler error 
        LOGI("[shadowfog] invalid response from scheduler , do nothing , and error time is %d",server_error_number);

	if ( (server_error_number++) >  PERR_SERVRE_ERROR_NUMBER )
	{
	    LOGI("[shadowfog] invalid response from scheduler too many %d ,so exit ",PERR_SERVRE_ERROR_NUMBER);
	    exit(EXIT_FAILURE);
	}
    	return ret;
    }

    //  then normal
    server_error_number = 0;
    shadowfog_speed_info.speed_update = 0;
    //////////////////////////////////////////////////////////////////////////////////////////

    json_t *obj      = NULL;
    json_t *_value   = NULL;
    json_t *tmp_obj  = NULL;
    const char *_key = NULL;
    json_error_t error;

    obj = json_loads(shadowfog_schedule.prs_buf, 0, &error);

    json_object_foreach(obj, _key, _value)
    {
        tmp_obj = json_object_get(obj, _key);
        if (strcmp("port_password", _key) == 0)
        {
            json_object_foreach(tmp_obj, _key, _value)
            {
                strcpy(shadowfog_schedule.prs_port_passwd[i].port, _key);
                strcpy(shadowfog_schedule.prs_port_passwd[i].passwd, json_string_value(_value));
                shadowfog_schedule.prs_num++;
            }
	        break;
        }
    }
    json_decref(obj);

    LOGI("[shadowfog] port number is %d", shadowfog_schedule.prs_num);

    for (i = 0; i < shadowfog_schedule.prs_num; i++) {
        LOGI("[shadowfog]  remove port : %s from scheduler ", shadowfog_schedule.prs_port_passwd[i].port);
        shadowfog_mapping_port_free(shadowfog_schedule.prs_port_passwd[i].port);
        remove_server(working_dir, shadowfog_schedule.prs_port_passwd[i].port);
        ret++;
    }

    shadowfog_schedule.prs_num = 0;

    return ret;
}

#if 0
static void
shadowfog_parse_user (jconf_t *conf)
{
    int i =0;
    //struct server server;
    if (conf != NULL) {
        for (i = 0; i < conf->port_password_num; i++) {
            struct server *server = (struct server *)malloc(sizeof(struct server));
            strncpy(server->port, conf->port_password[i].port, 8);
            strncpy(server->password, conf->port_password[i].password, 128);
	     server->traffic=0;
	     LOGI("[shadowfog] port : %s & password : %s ",conf->port_password[i].port,conf->port_password[i].password);
	     remove_server(working_dir, server->port);
            add_server(&manager, server);
        }
    }
    return;
}

void
shadowfog_get_user()
{
  return;
  char  path[256];
  char  cmd [256];
  char * shadowfog_ss_user_url = "";
  int  path_size = 256;
  //path = malloc(path_size);

  snprintf(path, path_size, "%s/.shadowfog_ss_user.json", working_dir);
  //LOGI ("[shadowfog] timeout working_dir is %s",working_dir);

  snprintf(cmd, path_size, "rm -rf %s", path);
   LOGI("[shadowfog] cmd: %s", cmd);

  if (system(cmd) == -1) {
        ERROR("[peer] rm json file");
  }


  memset(cmd,0x00,path_size);
  snprintf(cmd, path_size, "wget --no-cache -t 3 -O %s %s", path,shadowfog_ss_user_url);
  LOGI("[shadowfog] cmd: %s", cmd);

    if (system(cmd) == -1) {
        ERROR("[peer] wget");
    }

	  //ss_free(path);
  jconf_t *conf = NULL; 
  conf = read_jconf(path);
  shadowfog_parse_user(conf);
}
#endif

// sprintf(cmd ,"curl -X POST  -d @%s  --header \"Content-Type: application/json\" http://localhost:8080/v1/fogs/channels", path);
//   -m, --max-time SECONDS  Maximum time allowed for the transfer
//  --connect-timeout SECONDS  Maximum time allowed for connection

void shadowfog_init()
{
    int speed_times = 0;

    shadowfog_fog_info_init();
    shadowfog_child_init();
    shadowfog_enable_fastopen();
    shadowfog_get_host(shadowfog_real_host_address, shadowfog_host_address, 3, "shadowfog_host");

    #if PEAR_ENABLE_CHANGE_PASSWD
    shadow_change_password();
    #endif

    /* avoid malloc in refork blocked*/
    sigset_t block;
    sigemptyset(&block);
    sigaddset(&block, SIGCHLD);
    sigaddset(&block, SIGALRM);
    sigprocmask(SIG_BLOCK, &block, NULL);
 
    shadowfog_speed_init();

    for (speed_times = 0; speed_times < 3; speed_times++)
    {
        shadowfog_get_speed();
        sleep(1);
	}

    sigprocmask(SIG_UNBLOCK, &block, NULL);

#if 0
   char * args []= { "upnpc-static","-d",NULL,"TCP"};
   char  tempport[256];
   args[2]=tempport;
   for(int i=0;i<65535;i++)
   {
   	   sprintf(tempport,"%i",i);
   	   shadow_operate_upnp(sizeof(args)/sizeof(args[0]),args);
   }
#endif 

     return;
}
