#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>
#include <time.h>
#include <stdint.h>


#include "shadowfog_log.h"
#include "shadowfog_json.h"
#include "shadowfog_check.h"
#include "shadowfog_mapping.h"

#if USE_MAPPING
#include "shadowfog_upnpc.h"
#include "shadowfog_natpmpc.h"
#endif

/* For APP ssh & nomachine
 *
 * Added 8888 for KODExplorer
 *
 */
char * mapping_for_app []= {"4422","24422","4000","8888","8899"};

shadowfog_mapping_info_t shadowfog_mapping_info;

int shadowfog_mapping_for_app()
{
#if USE_MAPPING
    int  i = 0;
    int  items = sizeof(mapping_for_app)/sizeof(mapping_for_app[0]);   
    char *port = NULL;

    if (shadowfog_mapping_info.is_public_ip == 1)
    {
        return 0;
    }
		
    for(i = 0; i < items; i++)
    {
        port = mapping_for_app[i] ;

    	LOGI("[shadowfog] mapping for app  %d : %s ", i, mapping_for_app[i] );
		
	    if (shadowfog_mapping_info.is_support_upnp)
	    {
	        shadow_upnp_add(port);
	        LOGI("[ShadowFog]  upnp mapping for app......");
	    }
		
	    if (shadowfog_mapping_info.is_support_pmp)
	    {
	        shadow_pmp_add(port);
	        LOGI("[shadowfog]  pmp mapping for app......");
	    }
    }
#endif
    return 0;
}

int shadowfog_mapping_port_alloc(char * port)
{
#if USE_MAPPING
    int r =0;

    if (shadowfog_mapping_info.is_public_ip)
    {
        return 0;
    }

    if (shadowfog_mapping_info.is_support_upnp)
    {

        r = shadow_upnp_add(port);
	    if (!r)
        {
            shadowfog_mapping_info.upnp = shadowfog_mapping_info.upnp + 1;
            return 0;
        }
        else
        {
            shadowfog_mapping_info.max_server = shadowfog_mapping_info.upnp;
            return -1;
        }
    }

    if (shadowfog_mapping_info.is_support_pmp)
    {
        r = shadow_pmp_add(port);
        if (!r)
        {
            shadowfog_mapping_info.pmp = shadowfog_mapping_info.pmp + 1;
            return 0;
        }
        else
        {
            shadowfog_mapping_info.max_server = shadowfog_mapping_info.pmp;
            return -1;
        }
    }

    // no mapping protocol support , nothing i can do for it 
    return 1;

#endif
    return 0;
}

int shadowfog_mapping_port_free(char * port)
{
#if USE_MAPPING
    if (shadowfog_mapping_info.is_public_ip == 1)
    {
        return 0;
    }

    if (shadowfog_mapping_info.is_support_upnp)
    {
        shadow_upnp_delete(port);
        shadowfog_mapping_info.upnp = shadowfog_mapping_info.upnp - 1;
        return 0;
    }
    if(shadowfog_mapping_info.is_support_pmp)
    {
        shadow_pmp_delete(port);
        shadowfog_mapping_info.pmp = shadowfog_mapping_info.pmp - 1;
    }

    return 1;
#endif
    return 0;
}


int shadowfog_mapping_port_free_conf(char * path)
{
#if USE_MAPPING
    char port[16]  = {0x00};
    char temp[256] = {0x00};

    LOGI("[ShadowFog] Mapping free port config file : %s", path);

    FILE *f = fopen(path, "r");
    if (f == NULL)
    {
        LOGE("unable to open config  file : %s ",path);
        return -1;
    }

    fread(temp, 1, 256, f);
    pclose(f);

    shadowfog_json_get_string(temp, "server_port", port);

    LOGI("[ShadowFog] upnp delete port is  : %s", port);

    shadowfog_mapping_port_free(port);

#endif
    return 0;
}


int shadowfog_mapping_init(char *extern_ip, char *local_ip)
{
    memset(&shadowfog_mapping_info, 0x00, sizeof(shadowfog_mapping_info));
    if (strcmp(extern_ip, local_ip) == 0 || shadowfog_check_nat(extern_ip) == 0)
    {
        shadowfog_mapping_info.is_public_ip = 1;
    }
    else
    {
        shadowfog_mapping_info.is_public_ip = 0;
    }

	LOGI("[shadowfog-mapping] shadowfog_check_nat result: %d", shadowfog_check_nat(extern_ip));

#if USE_MAPPING

    // Check upnp --> pmp -->  more other
    shadowfog_mapping_info.is_support_upnp = shadow_support_upnp();
    shadowfog_mapping_info.is_support_pmp  = shadow_support_pmp();

    if (shadowfog_mapping_info.is_support_upnp)
    {
         LOGI("[ShadowFog] Support upnp ......");
         return 1;
    }

    if (shadowfog_mapping_info.is_support_pmp)
    {
         LOGI("[ShadowFog] Support pmp ......");
         return 1;
    }
#else
    shadowfog_mapping_info.is_support_upnp = 0;
    shadowfog_mapping_info.is_support_pmp  = 0;
    LOGI("[ShadowFog] Shutdown mapping......");
#endif

    return 1;
}
