#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>
#include <time.h>

//#include "utils.h"
#include "shadowfog_log.h"
#include "shadowfog_manager.h"
#include "shadowfog_port.h"

#include <stdint.h>

unsigned char port_map[65535]={0x00};

// http://www.garykessler.net/library/bad_ports.html
//  1 TCP 48421->192.168.31.169:4593  'XL_MINER_5444(DCDN)' '' 0
//  2 TCP 48422->192.168.31.169:4594  'XL_MINER_5444(DCDN)' '' 0
//  3 UDP 48423->192.168.31.169:4693  'XL_MINER_5444(DCDN)' '' 0

// http://www.wilderssecurity.com/threads/list-of-dangerous-ports.262113/
// http://www.acunetix.com/blog/articles/danger-open-ports-trojan-trojan/


int port_blacklist[] = {
        23, 25, 31, 42, 69, 80, 443,
        113, 123, 135, 137, 138, 139, 389, 445, 465, 500, 520, 539, 587, 593, 993,
        1002, 1010, 1024, 1025, 1034, 1068, 1170,
        1234, 1433, 1434, 1444, 1720, 1723, 1871, 1981, 2001, 2023, 2745, 2869, 2989,
        3024, 3127, 3128, 3129, 3130, 3150, 3208, 3268, 3332, 3389, 3700,
        4000, 4331, 4334, 4444, 4510, 4557, 4950,
        5554, 5800, 5900,
        6129, 6346, 6400, 6667, 6669, 6670, 8080, 8888, 8899, 9995, 9996,
        10080, 12345, 12346, 16660, 17185, 18753,
        20034, 20432, 20433, 27374, 27444, 27665,
        30100, 31335, 31337, 33270, 33567, 33568, 40421, 48421, 48422, 48423, 60008, 65000,
        4422, 24422 \
};

enum AVAILABLE_PORT_ERR
{
    PORT_SOCKET_ERR  = -4,
    PORT_INUSE_ERR   = -3,
    PORT_PROCESS_ERR = -2,
    PORT_CLOSE_ERR   = -1
};

/* return 0 if port is available */
int is_port_available_tcp(unsigned short port)
{
    LOGI("test is_port_available_tcp %d",port);
    struct sockaddr_in serv_addr;
    int ret = 0;

    int sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if(sockfd < 0)
    {
        printf("socket error\n");
        ret = PORT_SOCKET_ERR;
        return ret;
    }

    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(port);
    if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0)
    {
        if(errno == EADDRINUSE)
        {
            ret = PORT_INUSE_ERR;
            printf("the port is not available. already to other process\n");
        }
        else
        {
            ret = PORT_PROCESS_ERR;
            printf("could not bind to process (%d) %s\n", errno, strerror(errno));
        }
    }

    if (close(sockfd) < 0)
    {
        ret = PORT_CLOSE_ERR;
        printf("did not close fd: %s\n", strerror(errno));
    }
    return ret;
}

int is_in_blacklist(int port)
{
    int i =0;
    int count = (int)(sizeof(port_blacklist)/sizeof(port_blacklist[0]));
    LOGI("[shadowfog] blacklist count is %d",count);
    for(i=0;i<count;i++)
    {
        if(port_blacklist[i]==port)
        {
             return 1;
        }
    }
    return 0;
}


unsigned short get_random_port_gi()
{
    srand(mbedtls_timing_hardclock());
    srand(time(NULL));
    int p;
    do
    {
        p = rand() % 65534;
	 if(p<1024)
	 	p=p+0x400;
	 if(is_in_blacklist((int)p))
	 {
	     LOGI("[shadowfog] port  %d is in blacklist",p);
	     continue;
	 }
	 if(is_port_available_tcp(p)==0)
	 {
	     break;
	 }
    }
    while (1);
    return p;
}

unsigned short get_random_port_edu()
{
    srand(mbedtls_timing_hardclock());
    srand(time(NULL));
    int p;
    do
    {
        p = (rand() % 1024) ;
	 //  Ephemeral port https://en.wikipedia.org/wiki/Ephemeral_port
	 // why
	 //The allocations are temporary and only valid for the duration of the communication session. After completion of the communication session. 
	 //After an OS specific completion timeout is reached, the ports become available for reuse.[note 1] Since the ports are used on a per request basis they are also called dynamic ports.
	 // The Internet Assigned Numbers Authority (IANA) suggests the range 49152 to 65535 (215+214 to 216−1) for dynamic or private ports.[1]
       // Many Linux kernels use the port range 32768 to 61000.[note 2] FreeBSD has used the IANA port range since release 4.6. 
       // Previous versions, including the Berkeley Software Distribution (BSD), use ports 1024 to 5000 as ephemeral ports.[2]
	 if(p==0 || (p>=32768))
	 {
	    continue;
	 }
	 if(is_in_blacklist((int)p))
	 {
	     LOGI("[shadowfog] port  %d is in blacklist",p);
	     continue;
	 }
	 if(is_port_available_tcp(p)==0)
	 {
	     break;
	 }
    }
    while (1);
    return p;
}

unsigned short get_random_port(enum PORT_NET_TYPE port_net_type)
{
    if (port_net_type == PORT_NET_TYPE_EDU)
    {
        return get_random_port_edu();
    } 

    if (port_net_type == PORT_NET_TYPE_GI)
    {
        return get_random_port_gi();
    } 

    if (port_net_type == PORT_NET_TYPE_DC)
    {
        return get_random_port_gi();
    } 

    return get_random_port_gi();
}


void test_all_ports()
{
    unsigned short start_port = 1, end_port = 65534;
    unsigned short i;
    int ret = 0;
    for (i = start_port; i <= end_port; ++i)
    {
        ret = is_port_available_tcp(i);
        printf("port %d: ret: %d\n", i, ret);
    }
}

#if 0
int main(int argc, char **argv)
{
    unsigned p;
    p = get_random_port();
    printf("assigned a random port: %u\n", p);
}
#endif
